@@ -15,7 +15,30 @@ from .persistence import (
 logger = logging.getLogger(__name__)
 
 class PersistedTokenCache(msal.SerializableTokenCache):
-    """A token cache using given persistence layer, coordinated by a file lock."""
+    """A token cache backed by a persistence layer, coordinated by a file lock,
+    to sustain a certain level of multi-process concurrency for a desktop app.
+
+    The scenario is that multiple instances of same desktop app
+    (or even multiple different apps)
+    create their own ``PersistedTokenCache`` instances,
+    which are all backed by the same token cache file on disk
+    (known as a persistence). The goal is to have Single Sign On (SSO).
+
+    Each instance of ``PersistedTokenCache`` holds a snapshot of the token cache
+    in memory.
+    Each :func:`~find` call will
+    automatically reload token cache from the persistence when necessary,
+    so that it will have fresh data.
+    Each :func:`~modify` call will
+    automatically reload token cache from the persistence when necessary,
+    so that new writes will be appended on top of latest token cache data,
+    and then the new data will be immediately flushed back to the persistence.
+
+    Note: :func:`~deserialize` and :func:`~serialize` remain the same
+    as their counterparts in the parent class ``msal.SerializableTokenCache``.
+    In other words, they do not have the "reload from persistence if necessary"
+    nor the "flush back to persistence" behavior.
+    """
 
     def __init__(self, persistence, lock_location=None):
         super(PersistedTokenCache, self).__init__()
@@ -50,9 +73,21 @@ class PersistedTokenCache(msal.SerializableTokenCache):
             self._last_sync = time.time()
 
     def find(self, credential_type, **kwargs):  # pylint: disable=arguments-differ
-        with CrossPlatLock(self._lock_location):
-            self._reload_if_necessary()
-            return super(PersistedTokenCache, self).find(credential_type, **kwargs)
+        # Use optimistic locking rather than CrossPlatLock(self._lock_location)
+        retry = 3
+        for attempt in range(1, retry + 1):
+            try:
+                self._reload_if_necessary()
+            except Exception:  # pylint: disable=broad-except
+                # Presumably other processes are writing the file, causing dirty read
+                if attempt < retry:
+                    logger.debug("Unable to load token cache file in No. %d attempt", attempt)
+                    time.sleep(0.5)
+                else:
+                    raise  # End of retry. Re-raise the exception as-is.
+            else:  # If reload encountered no error, the data is considered intact
+                return super(PersistedTokenCache, self).find(credential_type, **kwargs)
+        return []  # Not really reachable here. Just to keep pylint happy.
 
 
 class FileTokenCache(PersistedTokenCache):
